
# This file was automatically generated by Dist::Zilla::Plugin::ModuleBuild v6.037
use strict;
use warnings;

use Module::Build 0.28;


my %module_build_args = (
  "build_requires" => {
    "Module::Build" => "0.28"
  },
  "configure_requires" => {
    "Module::Build" => "0.28"
  },
  "dist_abstract" => "Yet another testing framework",
  "dist_author" => [
    "Branislav Zahradn\x{ed}k <barney.cpan\@gmail.com>"
  ],
  "dist_name" => "Test-YAFT",
  "dist_version" => "1.0.4",
  "license" => "artistic_2",
  "module_name" => "Test::YAFT",
  "recursive_test_files" => 1,
  "requires" => {
    "Attribute::Handlers" => 0,
    "Context::Singleton" => 0,
    "Data::Dumper" => 0,
    "Devel::StackTrace" => 0,
    "Exporter::Tiny" => 0,
    "Module::Pluggable" => "4.8",
    "Ref::Util" => 0,
    "Safe::Isa" => 0,
    "Scalar::Util" => 0,
    "Sub::Install" => 0,
    "Sub::Override" => 0,
    "Sub::Util" => 0,
    "Syntax::Construct" => 0,
    "Test::Deep" => 0,
    "Test::Deep::Cmp" => 0,
    "Test::Differences" => 0,
    "Test::More" => "v0.970.0",
    "Test::Warnings" => "v0.38.0",
    "constant" => 0,
    "mro" => 0,
    "overload" => 0,
    "parent" => 0,
    "perl" => "5.014",
    "strict" => 0,
    "warnings" => 0
  },
  "test_requires" => {
    "HTTP::Response" => 0,
    "HTTP::Status" => 0,
    "JSON" => 0,
    "List::Util" => 0,
    "Test::EOL" => 0,
    "Test::Load::Helper" => 0,
    "Test::Tester" => "1.302107",
    "Type::Tiny" => 0,
    "Types::Common::Numeric" => 0,
    "feature" => 0
  }
);


my %fallback_build_requires = (
  "HTTP::Response" => 0,
  "HTTP::Status" => 0,
  "JSON" => 0,
  "List::Util" => 0,
  "Module::Build" => "0.28",
  "Test::EOL" => 0,
  "Test::Load::Helper" => 0,
  "Test::Tester" => "1.302107",
  "Type::Tiny" => 0,
  "Types::Common::Numeric" => 0,
  "feature" => 0
);


unless ( eval { Module::Build->VERSION(0.4004) } ) {
  delete $module_build_args{test_requires};
  $module_build_args{build_requires} = \%fallback_build_requires;
}

my $build = Module::Build->new(%module_build_args);


$build->create_build_script;
