use strict;
use warnings;

# this test was generated with Dist::Zilla::Plugin::Test::EOL 0.19

use Test::More 0.88;
use Test::EOL;

my @files = (
    'lib/Test/YAFT.pm',
    'lib/Test/YAFT.pod',
    'lib/Test/YAFT/Act.pm',
    'lib/Test/YAFT/Act/Context.pm',
    'lib/Test/YAFT/Argument.pm',
    'lib/Test/YAFT/Argument/Arrange.pm',
    'lib/Test/YAFT/Argument/Array.pm',
    'lib/Test/YAFT/Argument/Got.pm',
    'lib/Test/YAFT/Argument/Scalar.pm',
    'lib/Test/YAFT/Attributes.pm',
    'lib/Test/YAFT/Cmp.pm',
    'lib/Test/YAFT/Cmp/Compare.pm',
    'lib/Test/YAFT/Cmp/Complement.pm',
    'lib/Test/YAFT/Dumper.pm',
    'lib/Test/YAFT/Introduction.pod',
    'lib/Test/YAFT/Test/Deep.pod',
    'lib/Test/YAFT/Test/Exception.pod',
    'lib/Test/YAFT/Test/More.pod',
    'lib/Test/YAFT/Test/Spec.pod',
    'lib/Test/YAFT/Test/Warnings.pod',
    't/assumption/assumptions-with-got.t',
    't/assumption/assumptions.t',
    't/assumption/fail.t',
    't/assumption/had-no-warnings.t',
    't/assumption/nok.t',
    't/assumption/ok.t',
    't/assumption/pass.t',
    't/core/cmp-details.t',
    't/core/cmp-overload.t',
    't/core/internals-build-got.t',
    't/core/test-yaft-act-context.t',
    't/core/test-yaft-act.t',
    't/examples/test-rest-countries/resource-countries.t',
    't/examples/test-rest-countries/test-helper.pl',
    't/examples/test-type-tiny-constraint/constraint-test.t',
    't/examples/test-type-tiny-constraint/test-helper.pl',
    't/expectation/expect-all.t',
    't/expectation/expect-any.t',
    't/expectation/expect-array-each.t',
    't/expectation/expect-array-elements-only.t',
    't/expectation/expect-array-length-only.t',
    't/expectation/expect-array-length.t',
    't/expectation/expect-array.t',
    't/expectation/expect-bag.t',
    't/expectation/expect-blessed.t',
    't/expectation/expect-bool.t',
    't/expectation/expect-code.t',
    't/expectation/expect-compare.t',
    't/expectation/expect-complement.t',
    't/expectation/expect-false.t',
    't/expectation/expect-hash-each.t',
    't/expectation/expect-hash-keys-only.t',
    't/expectation/expect-hash-keys.t',
    't/expectation/expect-hash.t',
    't/expectation/expect-isa.t',
    't/expectation/expect-listmethods.t',
    't/expectation/expect-methods.t',
    't/expectation/expect-no-class.t',
    't/expectation/expect-none-of.t',
    't/expectation/expect-none.t',
    't/expectation/expect-num.t',
    't/expectation/expect-obj-isa.t',
    't/expectation/expect-re.t',
    't/expectation/expect-ref-type.t',
    't/expectation/expect-regexp-matches.t',
    't/expectation/expect-regexp-only.t',
    't/expectation/expect-regexpref-only.t',
    't/expectation/expect-regexpref.t',
    't/expectation/expect-scalarref-only.t',
    't/expectation/expect-scalarref.t',
    't/expectation/expect-set.t',
    't/expectation/expect-shallow.t',
    't/expectation/expect-str.t',
    't/expectation/expect-subbag-of.t',
    't/expectation/expect-subbag.t',
    't/expectation/expect-subhash-of.t',
    't/expectation/expect-subhash.t',
    't/expectation/expect-subset-of.t',
    't/expectation/expect-subset.t',
    't/expectation/expect-superbag-of.t',
    't/expectation/expect-superbag.t',
    't/expectation/expect-superhash-of.t',
    't/expectation/expect-superhash.t',
    't/expectation/expect-superset-of.t',
    't/expectation/expect-superset.t',
    't/expectation/expect-true.t',
    't/expectation/expect-use-class.t',
    't/expectation/expect-value.t',
    't/expectation/ignore.t',
    't/foundation/deep-diag.t',
    't/foundation/eq-deeply.t',
    't/foundation/test-deep-cmp.t',
    't/foundation/test-frame.t',
    't/test-helper.pl',
    't/util/act.t',
    't/util/arrange.t',
    't/util/bail-out.t',
    't/util/diag.t',
    't/util/done-testing.t',
    't/util/explain.t',
    't/util/note.t',
    't/util/plan.t',
    't/util/skip.t',
    't/util/subtest.t',
    't/util/todo-skip.t'
);

eol_unix_ok($_, { trailing_whitespace => 1 }) foreach @files;
done_testing;
